@@ -46,6 +46,8 @@
 #include <gnome-keyring.h>
 #endif
 
+G_DEFINE_TYPE (VinoServer, vino_server, G_TYPE_OBJECT);
+
 /* If an authentication attempt failes, delay the next
  * authentication attempt for 5 seconds.
  */
@@ -60,7 +62,9 @@ struct _VinoServerPrivate
   VinoCursorData   *cursor_data;
   VinoPrompt       *prompt;
   VinoStatusIcon   *icon;
+  gboolean          display_status_icon;
   VinoDBusListener *listener;
+  gboolean          use_dbus_listener;
   VinoUpnp         *upnp;
 
   GIOChannel       *io_channel[RFB_MAX_SOCKETLISTEN];
@@ -87,6 +91,7 @@ struct _VinoServerPrivate
   guint             lock_screen : 1;
   guint             disable_background : 1;
   guint             use_upnp : 1;
+  guint             disable_xdamage : 1;
 };
 
 struct _VinoClient
@@ -109,6 +114,8 @@ enum
   PROP_ON_HOLD,
   PROP_PROMPT_ENABLED,
   PROP_VIEW_ONLY,
+  PROP_DISPLAY_STATUS_ICON,
+  PROP_USE_DBUS_LISTENER,
   PROP_NETWORK_INTERFACE,
   PROP_USE_ALTERNATIVE_PORT,
   PROP_ALTERNATIVE_PORT,
@@ -118,7 +125,8 @@ enum
   PROP_PORT,
   PROP_LOCK_SCREEN,
   PROP_DISABLE_BACKGROUND,
-  PROP_USE_UPNP
+  PROP_USE_UPNP,
+  PROP_DISABLE_XDAMAGE
 };
 
 static enum rfbNewClientAction vino_server_auth_client (VinoServer *server,
@@ -270,11 +278,35 @@ vino_server_get_use_upnp (VinoServer *server)
   return server->priv->use_upnp;
 }
 
+void
+vino_server_set_disable_xdamage (VinoServer *server,
+                                 gboolean disable_xdamage)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  disable_xdamage = disable_xdamage != FALSE;
+
+  if (server->priv->disable_xdamage != disable_xdamage)
+    {
+      server->priv->disable_xdamage = disable_xdamage;
+      g_object_notify (G_OBJECT (server), "disable-xdamage");
+    }
+}
+
+gboolean
+vino_server_get_disable_xdamage (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->disable_xdamage;
+}
+
 static void
 vino_server_client_accepted (VinoServer *server,
                              VinoClient *client)
 {
-  vino_status_icon_add_client (server->priv->icon, client);
+  if (server->priv->display_status_icon)
+    vino_status_icon_add_client (server->priv->icon, client);
 
   vino_server_unlock_screen ();
 
@@ -287,12 +319,15 @@ static void
 vino_server_client_disconnected (VinoServer *server,
                                  VinoClient *client)
 {
-  if (vino_status_icon_remove_client (server->priv->icon, client))
+  if (server->priv->display_status_icon)
     {
-      vino_server_lock_screen (server);
+      if (vino_status_icon_remove_client (server->priv->icon, client))
+        {
+          vino_server_lock_screen (server);
 
-      if (vino_server_get_disable_background (server))
-	vino_background_draw (TRUE);
+          if (vino_server_get_disable_background (server))
+            vino_background_draw (TRUE);
+        }
     }
 }
 
@@ -637,9 +672,9 @@ vino_server_defer_client_auth (VinoServer *server,
 
   vino_server_set_client_on_hold (server, client, TRUE);
 
-  client->auth_timeout = g_timeout_add (VINO_SERVER_AUTH_DEFER_LEN * 1000,
-					(GSourceFunc) vino_server_auth_client_deferred,
-					client);
+  client->auth_timeout = g_timeout_add_seconds (VINO_SERVER_AUTH_DEFER_LEN,
+                                                (GSourceFunc) vino_server_auth_client_deferred,
+                                                client);
 }
 
 static char *
@@ -849,7 +884,7 @@ vino_server_setup_framebuffer (VinoServer *server)
   g_return_if_fail (server->priv->fb == NULL);
   g_return_if_fail (server->priv->cursor_data == NULL);
 
-  server->priv->fb = vino_fb_new (server->priv->screen);
+  server->priv->fb = vino_fb_new (server->priv->screen, server->priv->disable_xdamage);
 
   g_signal_connect_swapped (server->priv->fb, "size-changed",
 			    G_CALLBACK (vino_server_screen_size_changed),
@@ -1018,8 +1053,6 @@ vino_server_init_from_screen (VinoServer *server,
 
   vino_mdns_add_service ("_rfb._tcp", rfb_screen->rfbPort);
 
-  server->priv->listener = vino_dbus_listener_new (server);
-
   cb = gtk_clipboard_get_for_display (gdk_screen_get_display (screen),
                                       GDK_SELECTION_CLIPBOARD);
   g_signal_connect (cb,
@@ -1027,8 +1060,8 @@ vino_server_init_from_screen (VinoServer *server,
                     G_CALLBACK (vino_server_clipboard_cb),
                     server);
 
-  server->priv->icon = vino_status_icon_new (server, server->priv->screen);
   server->priv->upnp = NULL;
+  server->priv->disable_xdamage = FALSE;
 }
 
 static void
@@ -1128,6 +1161,12 @@ vino_server_set_property (GObject      *object,
     case PROP_VIEW_ONLY:
       vino_server_set_view_only (server, g_value_get_boolean (value));
       break;
+    case PROP_DISPLAY_STATUS_ICON:
+      vino_server_set_display_status_icon (server, g_value_get_boolean (value));
+      break;
+    case PROP_USE_DBUS_LISTENER:
+      vino_server_set_use_dbus_listener (server, g_value_get_boolean (value));
+      break;
     case PROP_NETWORK_INTERFACE:
       vino_server_set_network_interface (server, g_value_get_string (value));
       break;
@@ -1155,6 +1194,9 @@ vino_server_set_property (GObject      *object,
     case PROP_USE_UPNP:
       vino_server_set_use_upnp (server, g_value_get_boolean (value));
       break;
+    case PROP_DISABLE_XDAMAGE:
+      vino_server_set_disable_xdamage (server, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1183,6 +1225,12 @@ vino_server_get_property (GObject    *object,
     case PROP_VIEW_ONLY:
       g_value_set_boolean (value, server->priv->view_only);
       break;
+    case PROP_DISPLAY_STATUS_ICON:
+      g_value_set_boolean (value, server->priv->display_status_icon);
+      break;
+    case PROP_USE_DBUS_LISTENER:
+      g_value_set_boolean (value, server->priv->use_dbus_listener);
+      break;
     case PROP_NETWORK_INTERFACE:
       g_value_set_string (value, server->priv->network_interface);
       break;
@@ -1213,6 +1261,9 @@ vino_server_get_property (GObject    *object,
     case PROP_USE_UPNP:
       g_value_set_boolean (value, server->priv->use_upnp);
       break;
+    case PROP_DISABLE_XDAMAGE:
+      g_value_set_boolean (value, server->priv->disable_xdamage);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1220,7 +1271,24 @@ vino_server_get_property (GObject    *object,
 }
 
 static void
-vino_server_instance_init (VinoServer *server)
+vino_server_constructed (GObject *object)
+{
+  VinoServer *server = VINO_SERVER (object);
+
+  if (server->priv->use_dbus_listener)
+    server->priv->listener = vino_dbus_listener_new (server);
+  else
+    server->priv->listener = NULL;
+
+  if (server->priv->display_status_icon)
+    server->priv->icon = vino_status_icon_new (server, server->priv->screen);
+  else
+    server->priv->icon = NULL;
+
+}
+
+static void
+vino_server_init (VinoServer *server)
 {
   server->priv = g_new0 (VinoServerPrivate, 1);
 }
@@ -1232,6 +1300,7 @@ vino_server_class_init (VinoServerClass *klass)
   
   parent_class = g_type_class_peek_parent (klass);
   
+  gobject_class->constructed  = vino_server_constructed;
   gobject_class->finalize     = vino_server_finalize;
   gobject_class->set_property = vino_server_set_property;
   gobject_class->get_property = vino_server_get_property;
@@ -1284,6 +1353,27 @@ vino_server_class_init (VinoServerClass *klass)
                                                          G_PARAM_STATIC_NICK |
                                                          G_PARAM_STATIC_BLURB));
 
+   g_object_class_install_property (gobject_class,
+				   PROP_DISPLAY_STATUS_ICON,
+				   g_param_spec_boolean ("display-status-icon",
+							 "Display the status icon",
+							 "Allow to display the status icon",
+							 TRUE,
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_STRINGS));
+
+   g_object_class_install_property (gobject_class,
+				   PROP_USE_DBUS_LISTENER,
+				   g_param_spec_boolean ("use-dbus-listener",
+							 "Use the dbus listener",
+							 "Allow to use the dbus listener",
+							 TRUE,
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_STRINGS));
+
+
   g_object_class_install_property (gobject_class,
 				   PROP_NETWORK_INTERFACE,
 				   g_param_spec_string ("network-interface",
@@ -1409,37 +1499,20 @@ vino_server_class_init (VinoServerClass *klass)
                                                          G_PARAM_STATIC_NICK |
                                                          G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (gobject_class,
+				   PROP_DISABLE_XDAMAGE,
+				   g_param_spec_boolean ("disable-xdamage",
+							 "Disable XDamage",
+							 "Whether to disable XDamage",
+							 FALSE,
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
+
 }
 
-GType
-vino_server_get_type (void)
-{
-  static GType object_type = 0;
-
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-	{
-	  sizeof (VinoServerClass),
-	  (GBaseInitFunc) NULL,
-	  (GBaseFinalizeFunc) NULL,
-	  (GClassInitFunc) vino_server_class_init,
-	  NULL,           /* class_finalize */
-	  NULL,           /* class_data */
-	  sizeof (VinoServer),
-	  0,              /* n_preallocs */
-	  (GInstanceInitFunc) vino_server_instance_init,
-	};
-      
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "VinoServer",
-                                            &object_info, 0);
-    }
-
-  return object_type;
-}
-
-
 VinoServer *
 vino_server_new (GdkScreen *screen,
 		 gboolean   view_only)
@@ -1484,6 +1557,24 @@ vino_server_set_view_only (VinoServer *server,
     }
 }
 
+void
+vino_server_set_display_status_icon (VinoServer *server,
+    gboolean display_status_icon)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  server->priv->display_status_icon = display_status_icon;
+}
+
+void
+vino_server_set_use_dbus_listener (VinoServer *server,
+    gboolean use_dbus_listener)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  server->priv->use_dbus_listener = use_dbus_listener;
+}
+
 G_CONST_RETURN char *
 vino_server_get_network_interface (VinoServer *server)
 {
@@ -1545,6 +1636,7 @@ vino_server_set_use_alternative_port (VinoServer *server,
         }
 
       g_object_notify (G_OBJECT (server), "use-alternative-port");
+      g_object_notify (G_OBJECT (server), "alternative-port");
     }
 }
 
@@ -1773,6 +1865,16 @@ vino_server_get_port (VinoServer *server)
   return server->priv->rfb_screen->rfbPort;
 }
 
+int
+vino_server_get_external_port (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), 0);
+
+  return server->priv->use_upnp && VINO_IS_UPNP (server->priv->upnp) ?
+           vino_upnp_get_external_port (server->priv->upnp) :
+           server->priv->rfb_screen->rfbPort;
+}
+
 gboolean
 vino_server_get_lock_screen (VinoServer *server)
 {
@@ -1797,7 +1899,6 @@ vino_server_set_lock_screen (VinoServer *server,
     }
 }
 
-
 VinoStatusIcon *
 vino_server_get_status_icon (VinoServer *server)
 {
